'**** A P P L I C A T I O N   N O T E   ************************************
'*
'* PHOENIXstudios PC_DIMMER Hardware-Interface-Firmware
'* Copyright (c) 2006-2018 by Dr.-Ing. Christian Nding
'* Last change 26.03.2018
'*
'* Chip-Typ: Atmel AVR ATmega8
'*
'*
'* Updates: http://www.pcdimmer.de
'*
'***************************************************************************
' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version2 of
' the License, or (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
' General Public License for more details.
'
' If you have no copy of the GNU General Public License, write to the
' Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
'***************************************************************************
'
'PinD0 RS232-In
'PinD1 RS232-Out
'
'PinD2 Szene: Arbeitslicht
'PinD3 Szene: TV
'PinD4 Szene: Aus
'
'PinD5 Up/Down: Linestras
'PinD6 Up/Down: Lichterketten
'PinD7 Up/Down: Esstisch
'PinB0 RC5 Decoder
'
'PinB1 FREI
'PinB2 FREI
'
'PinB3 Up/Down: Regal
'PinB4 Up/Down: Treppe
'PinB5 Up/Down: Sofa
'
'PinB6 XTAL
'PinB7 XTAL
'
'PinC0 Fader
'PinC1 FREI
'PinC2 FREI
'PinC3 FREI
'PinC4 FREI
'PinC5 FREI
'
'PinC6 RESET
'***************************************************************************

'Definitionen

$regfile = "m88def.dat"
$crystal = 7372800
$baud = 115200
$hwstack = 64
$swstack = 30
$framesize = 120

$lib "mcsbyte.lbx"

'Programmkonstanten
Const Lastchannel = 8                                       'Letzter Kanal im LCD-Men

'###############################################################################
'Variablen definieren

' Variablen fr RS232-Interrupt
Dim Rs232_inframe(6) As Byte
Dim Rs232_incounter As Byte
Dim Rs232_inchannel As Byte
Dim Rs232_invalue As Byte
Dim Rs232_outframe(6) As Byte

Dim Channelvalue(lastchannel) As Byte
Dim Channelvalue_old(lastchannel) As Byte

Dim Temp As Byte
Dim Temp2 As Single
Dim T As Single
Dim Stp As Byte
Dim Ftime As Long
Dim Nsteps As Integer

Dim I As Byte

Dim Buttondirection1 As Bit
Dim Buttondirection2 As Bit
Dim Buttondirection3 As Bit
Dim Buttondirection4 As Bit
Dim Buttondirection5 As Bit
Dim Buttondirection6 As Bit
'Dim Buttondirection7 As Bit
'Dim Buttondirection8 As Bit
'Dim Buttondirection9 As Bit

Dim Upperborder As Integer
Dim Lowerborder As Integer
Dim Faderonvalue As Bit
Dim Fadervalue As Word

Dim Rc5address As Byte
Dim Rc5command As Byte

'Funktionen definieren
Declare Sub Sendrs232(byval Channel As Byte , Byval Startvalue As Byte , Byval Endvalue As Byte , Byval Fadetime As Long)
Declare Sub Increasechannel(byval Channel As Byte)
Declare Sub Decreasechannel(byval Channel As Byte)

'****************** Startup ******************
   On Urxc Rec_isr                                          'define serial receive ISR
   Enable Urxc                                              'Serial RX complete interrupt

   Config Adc = Single , Prescaler = Auto                   'Analog-Fader initiieren
   Start Adc

   Config Portb = Input
   Config Pinc.0 = Input
   Config Portd = Input
   Config Pind.1 = Output

   Config Rc5 = Pinb.0 Wait = 2000
   Config Timer1 = Timer , Prescale = 1
   'Config Rc5 = Pinb.0 , Timer = 1 , Mode = Background

   Btn_1 Alias Pind.5
   Btn_2 Alias Pind.7
   Btn_3 Alias Pinb.4
   Btn_4 Alias Pinb.2
   Btn_5 Alias Pind.6
   Btn_6 Alias Pinb.3
   Btn_7 Alias Pind.2
   Btn_8 Alias Pind.3
   Btn_9 Alias Pind.4

   Portb = &B11111111
   Portc = &B11111111
   Portd = &B11111111

   I = 0
   Rs232_incounter = 0

   Enable Interrupts                                        'enable interrupts to occur

'****************** Programmschleife ******************
   Do
      'check for key-press
      If Btn_1 = 0 Then Gosub Fct1                          ' Linestras
      If Btn_2 = 0 Then Gosub Fct2                          ' Esstisch
      If Btn_3 = 0 Then Gosub Fct3                          ' Regalbeleuchtung
      If Btn_4 = 0 Then Gosub Fct4                          ' Sofa
      If Btn_5 = 0 Then Gosub Fct5                          ' Lichterketten
      If Btn_6 = 0 Then Gosub Fct6                          ' Treppenbeleuchtung
      If Btn_7 = 0 Then Gosub Fct7                          ' Szene: Arbeitslicht
      If Btn_8 = 0 Then Gosub Fct8                          ' Szene: Fernsehlicht
      If Btn_9 = 0 Then Gosub Fct9                          ' Szene: Blackout

      ' read fader on analog-input
      Gosub Readfader

      ' read RC5-receiver and set channels / start scenes
      Getrc5(rc5address , Rc5command)
      'If _rc5_bits.4 = 1 Then                                   ' if there is RC5 code received
      '    _rc5_bits.4 = 0                                       ' you MUST reset this flag in order to receive a new
      '   Rc5address = Rc5_address                         ' Address
      '   Rc5command = Rc5_command                         ' Command
      'end if

      If Rc5address = 11 Then
         Rc5command = Rc5command And &B01111111         ' ignore toggle-bit

         Select Case Rc5command
            Case 1
            Case 2
            Case 3
            Case 4
            Case 5
            Case 6
            Case 7
               ' Szene: Arbeitslicht
               Gosub Fct7
            Case 8
               ' Szene: Fernsehlicht
               Gosub Fct8
            Case 9
               ' Szene: Blackout
               Gosub Fct9
            Case 112
               ' Szene: Fernsehlicht
               Gosub Fct8
            Case 113
               ' Sofalicht 100%
               Channelvalue_old(7) = Channelvalue(7)
               Channelvalue(7) = 255
               Call Sendrs232(7 , Channelvalue_old(7) , Channelvalue(7) , 3000)
            Case 114
               ' Sofalicht 50%
               Channelvalue_old(7) = Channelvalue(7)
               Channelvalue(7) = 128
               Call Sendrs232(7 , Channelvalue_old(7) , Channelvalue(7) , 3000)
            Case 115
               ' Szene: Blackout
               Gosub Fct9
            Case Else
         End Select

         ' ignore other commands for 2 seconds
         wait 2
      End If
   Loop

Fct1:
   'Linestras
   Toggle Buttondirection1

   While Btn_1 = 0
      If Buttondirection1 = 1 Then
         Call Increasechannel(1)
         Call Increasechannel(2)
      Else
         Call Decreasechannel(1)
         Call Decreasechannel(2)
      End If
      Waitms 40
   Wend
Return

Fct2:
   ' Esstisch
   Toggle Buttondirection2

   While Btn_2 = 0
      If Buttondirection2 = 1 Then
         Call Increasechannel(5)
      Else
         Call Decreasechannel(5)
      End If
      Waitms 40
   Wend
Return

Fct3:
   ' Regalbeleuchtung
   Toggle Buttondirection3

   While Btn_3 = 0
      If Buttondirection3 = 1 Then
         Call Increasechannel(4)
      Else
         Call Decreasechannel(4)
      End If
      Waitms 40
   Wend
Return

Fct4:
   ' Sofa
   Toggle Buttondirection4

   While Btn_4 = 0
      Reset Faderonvalue

      If Buttondirection4 = 1 Then
         Call Increasechannel(7)
      Else
         Call Decreasechannel(7)
      End If

      Waitms 40
   Wend
Return

Fct5:
   ' Lichterketten
   Toggle Buttondirection5

   While Btn_5 = 0
      If Buttondirection5 = 1 Then
         Call Increasechannel(3)
      Else
         Call Decreasechannel(3)
      End If
      Waitms 40
   Wend
Return

Fct6:
   ' Treppenbeleuchtung
   Toggle Buttondirection6

   While Btn_6 = 0
      If Buttondirection6 = 1 Then
         Call Increasechannel(8)
      Else
         Call Decreasechannel(8)
      End If
      Waitms 40
   Wend
Return

Fct7:
   ' Arbeitslicht
   For I = 1 To 7
      Channelvalue_old(i) = Channelvalue(i)
   Next I

   Channelvalue(1) = 255
   Channelvalue(2) = 255
   Channelvalue(3) = 0
   Channelvalue(4) = 255
   Channelvalue(5) = 255
   Channelvalue(6) = 0
   Channelvalue(7) = 255

   Reset Faderonvalue

   For I = 1 To 7
      Call Sendrs232(i , Channelvalue_old(i) , Channelvalue(i) , 3000)
   Next I

   While Btn_7 = 0
      Waitms 150
   Wend

   ' Da nun Arbeitslicht in jedem Fall mit nchstem Tastendruck ausblenden!
   Set Buttondirection1
   Set Buttondirection2
   Set Buttondirection3
   Set Buttondirection4
   Reset Buttondirection5                                   ' einblenden
   Set Buttondirection6
Return

Fct8:
   ' Fernsehlicht
   For I = 1 To 7
      Channelvalue_old(i) = Channelvalue(i)
   Next I

   Channelvalue(1) = 0
   Channelvalue(2) = 0
   Channelvalue(3) = 128
   Channelvalue(4) = 64
   Channelvalue(5) = 0
   Channelvalue(6) = 0
   Channelvalue(7) = 32

   Reset Faderonvalue

   For I = 1 To 7
      Call Sendrs232(i , Channelvalue_old(i) , Channelvalue(i) , 3000)
   Next I

   While Btn_8 = 0
      Waitms 150
   Wend

   ' Da nun Blackout in jedem Fall mit nchstem Tastendruck einblenden!
   Reset Buttondirection1                                   ' einblenden
   Reset Buttondirection2                                   ' einblenden
   Reset Buttondirection3
   Reset Buttondirection4
   Reset Buttondirection5
   Reset Buttondirection6
Return

Fct9:
   ' Blackout
   Reset Faderonvalue

   For I = 1 To 7
      Channelvalue_old(i) = Channelvalue(i)
      Channelvalue(i) = 0
      Call Sendrs232(i , Channelvalue_old(i) , Channelvalue(i) , 3000)
   Next I

   While Btn_9 = 0
      Waitms 150
   Wend

   ' Da nun Blackout in jedem Fall mit nchstem Tastendruck einblenden!
   Reset Buttondirection1
   Reset Buttondirection2
   Reset Buttondirection3
   Reset Buttondirection4
   Reset Buttondirection5
   Reset Buttondirection6
Return


Readfader:
   ' Fader einlesen
   Fadervalue = Getadc(0)
   Shift Fadervalue , Right , 2                             '10-Bit-Wert zu 8-Bit-Wert ndern (0..1023 -> 0..255)
   Fadervalue = 255 - Fadervalue

   Upperborder = Channelvalue(7) + 5
   Lowerborder = Channelvalue(7) - 5

   If Faderonvalue = 0 Then
      ' Wert von Kanal 7 wurde zwischenzeitlich von Szene oder Button verndert -> prfen, ob Fader innerhalb der Grenzen liegt

      If Fadervalue > Lowerborder And Fadervalue < Upperborder Then
         ' Fader ist innerhalb der Grenzen -> neuen Wert bernehmen
         Set Faderonvalue
         Channelvalue(7) = Fadervalue
         Call Sendrs232(7 , Channelvalue(7) , Channelvalue(7) , 0)
      End If
   Else
      ' Fader sollte auf korrektem Wert sein - Werte aber nur senden, wenn auerhalb des Safemargins
      If Fadervalue < Lowerborder Or Fadervalue > Upperborder Then
         Channelvalue(7) = Fadervalue
         Call Sendrs232(7 , Channelvalue(7) , Channelvalue(7) , 0)
      End If
   End If
Return


Sub Increasechannel(byval Channel As Byte)
   If Channelvalue(channel) <= 250 Then
      Channelvalue(channel) = Channelvalue(channel) + 5
   Else
      Channelvalue(channel) = 255
   End If
   Call Sendrs232(channel , Channelvalue(channel) , Channelvalue(channel) , 0)
End Sub


Sub Decreasechannel(byval Channel As Byte)
   If Channelvalue(channel) >= 5 Then
      Channelvalue(channel) = Channelvalue(channel) - 5
   Else
      Channelvalue(channel) = 0
   End If
   Call Sendrs232(channel , Channelvalue(channel) , Channelvalue(channel) , 0)
End Sub


'****************** RS232-Funktionen ******************

' RS232 Senden
Sub Sendrs232(byval Channel As Byte , Byval Startvalue As Byte , Byval Endvalue As Byte , Byval Fadetime As Long)
   T = Fadetime / 10                                        '{100ms in 10ms umwandeln}
   Nsteps = Startvalue - Endvalue
   Nsteps = Abs(nsteps)

   Temp = 0

   If Nsteps <> 0 And T <> 0 Then
      Do
         Incr Stp
         Temp2 = T * Stp
         Temp2 = Temp2 / Nsteps
         Temp2 = Round(temp2)
         Ftime = Temp2
         Temp2 = Ftime * Nsteps
         Temp2 = Temp2 / Stp
         Temp2 = T - Temp2
         Temp2 = Temp2 * 100
         Temp2 = Temp2 / T
         Temp2 = Abs(temp2)
      Loop Until Temp2 <= 15 Or Stp >= 4
   Else
      Ftime = 0
      Incr Stp
   End If

   'RS232Frame mit Daten fllen
   '*************ADRESSE****************
   Temp = Channel + 127
      Rs232_outframe(1) = Temp                              ' Bit 1 = "High", der Rest Kanaladresse [0..127]
   '**************STEPS*****************
   Stp = Stp - 1
   Temp = 0
   Temp = Temp Or Stp
      Rs232_outframe(2) = Temp                              'Steps bis zum Endwert
   '*************FADE_1****************
   Temp = Low(ftime)
   Temp = Temp And 127
      Rs232_outframe(5) = Temp                              ' Fadezeit
   '*************FADE_2****************
   Shift Ftime , Right , 7
   Temp = Low(ftime)
   Temp = Temp And 127
      Rs232_outframe(4) = Temp                              ' Fadezeit
   '*************FADE_3****************
   Shift Ftime , Right , 7
   Temp = Low(ftime)
   Temp = Temp And 127
      Rs232_outframe(3) = Temp                              ' Fadezeit
   '*************ENDWERT***************
   Temp = 255 - Endvalue
   Temp = Temp / 2
   Temp = Temp And 127
      Rs232_outframe(6) = Temp                              ' Kanalendwert setzen [0..127]

   Printbin Rs232_outframe(1) ; 6
End Sub

' RS232-Empfang
Rec_isr:
If Usr.7 = 1 Then                                           'Nur wenn neue Daten empfangen wurden, diese auch verwerten
   Incr Rs232_incounter
   Rs232_inframe(rs232_incounter) = Udr

   ' Prfen, ob Startbit gesetzt ist
   If Rs232_inframe(1) < 128 Then
      ' Startbit fehlt -> Reset des Counters und Exit
      Rs232_incounter = 0
      Goto Error
   End If

   'Nun empfangene Daten verarbeiten
   If Rs232_incounter >= 6 Then
      Rs232_incounter = 0

      If Rs232_inframe(1) < 128 Then Goto Error             'Fehler im Datenpaket, wenn kein Startbit vorhanden
      If Rs232_inframe(6) > 127 Then Goto Error             'Fehler im Datenpaket, wenn Kanalendwert > 127

      Rs232_invalue = 127 - Rs232_inframe(6)                'Kanalwert berechnen (da nur 0..127 bertragen werden)
      Rs232_invalue = Rs232_invalue * 2
      Rs232_inchannel = Rs232_inframe(1) - 127              'Kanal herausfiltern (da Bit0 als Startbit)

      If Rs232_inchannel = 7 Then
         Reset Faderonvalue
      End If

      If Rs232_inchannel <= Lastchannel Then
         Channelvalue(rs232_inchannel) = Rs232_invalue
      End If
   End If
End If

Error:
Return

End                                                         'end program